"""
The MIT License (MIT)

Copyright (c) 2023-present scrazzz

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations

import time
from typing import Any, Dict, List, TYPE_CHECKING

import requests
import click
import discord
from rich.progress import track

from a2d.utils import (
    validate_thread_url,
    validate_webhook_url,
    normify_comment,
    construct_file,
)

if TYPE_CHECKING:
    POSTS = List[Dict[str, Any]]

def filter_posts(*, skip_comments: bool, skip_files: bool, posts: POSTS) -> POSTS:
    if (skip_comments and skip_files):
        # idk why you'd want this but it just returns an empty list
        return list(
            filter(lambda post: post.get('com') == None and post.get('filename') == None, posts)
        )
    if skip_comments:
        return list(filter(lambda post: post.get('com') == None or post.get('filename'), posts))
    if skip_files:
        return list(filter(lambda post: post.get('filename') == None, posts))
    else:
        return posts

@click.command()
@click.argument('thread')
@click.argument('webhook')
@click.option('--skip-comments', is_flag=True, default=False, help='Skips all comments')
@click.option('--skip-files', is_flag=True, default=False, help='Skips all comments containing files')
@click.option('--delay', type=click.INT, default=1, help='Delay to send webhooks', show_default=True)
def cmain(
    thread: str,
    webhook: str,
    skip_comments: bool,
    skip_files: bool,
    delay: int,
):
    thread_url = validate_thread_url(thread)
    webhook_url = validate_webhook_url(webhook)

    if thread_url['success'] == False:
        raise click.ClickException(f'{thread} is not a valid thread url')
    if webhook_url['success'] == False:
        raise click.ClickException(f'{webhook} is not a valid webhook url')

    wh = discord.SyncWebhook.from_url(webhook)

    thread_data = thread_url['result']
    r = requests.get(f'https://a.4cdn.org/{thread_data[2]}/thread/{thread_data[3]}.json')
    r.raise_for_status()
    js = r.json()

    posts = js['posts']
    thread_title = posts[0]['semantic_url']
    filtered = filter_posts(skip_comments=skip_comments, skip_files=skip_files, posts=posts)

    # map of thread number (no) to message (WebhookMessage)
    com_map: Dict[int, discord.SyncWebhookMessage] = {}

    for post in track(filtered):
        tim = post.get('tim')
        filename = post.get('filename')
        ext = post.get('ext')
        no = post['no']
        replyto = post.get('resto')

        # If the comment doesn't have an attachment
        if tim is None:
            com = post['com']
            # Check if the comment is a reply or not
            if replyto:
                # Hyperlinks don't work if either --skip-files or --skip-comments flag
                # is used because some comments may get skipped, so we don't hyperlink replies
                # because it has a chance of KeyError-ing when indexing `com_map`.
                if skip_comments or skip_files:
                    fmt = f'>>>{replyto}\n{normify_comment(com)}'[:2000]
                else:
                    replyto_hyperlink = f'[>>{replyto}]({com_map[replyto].jump_url})'
                    fmt = f'{replyto_hyperlink}{normify_comment(com)}'[:2000]

                sent = wh.send(
                    content=fmt,
                    username=f'{post["name"]} #{no}',
                    wait=True,
                    suppress_embeds=True
                )
            # If it reaches here, then it's a comment without a reply
            else:
                sent = wh.send(
                    content=str(post.get('com'))[:2000],
                    username=f'{post["name"]} #{no}',
                    wait=True,
                )
            com_map[no] = sent

        # If the comment has an attachment
        elif tim and filename and ext is not None:
            com = post.get('com')

            if com and replyto:
                fmt = f'>>>{replyto}\n{normify_comment(com)}'[:2000]
            elif com:
                fmt = f'{normify_comment(com)}'[:2000]
            else:
                fmt = ''

            sent = wh.send(
                content=fmt,
                username=f'{post["name"]} #{no}',
                file=construct_file(
                    board=thread_data[2],
                    tim=tim,
                    filename=filename,
                    ext=ext
                ),
                wait=True,
            )
            com_map[no] = sent

        time.sleep(delay)

if __name__ == '__main__':
    cmain()


import re
from setuptools import setup

with open('a2d/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE)[1] # type: ignore

with open('README.md', encoding='utf-8') as f:
    readme = f.read()

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name='a2d',
    author='scrazzz',
    url='https://github.com/scrazzz/a2d',
    description='Archive 4Chan threads to Discord',
    version=version,
    license='MIT',
    packages=['a2d'],
    long_description=readme,
    long_description_content_type='text/markdown',
    include_package_data=True,
    install_requires=requirements,
    python_requires='>=3.8.0',
    entry_points={
        'console_scripts': [
            'a2d = a2d.main:cmain',
        ],
    },
)


"""
The MIT License (MIT)

Copyright (c) 2023-present scrazzz

Permission is hereby granted, free of charge, to any person obtaining a
copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE.
"""

from __future__ import annotations

import re
import io
from typing import Any, Dict

import requests
import discord

THREAD_RE = re.compile(
    r'https:\/\/(?P<domain>boards.(4chan|4channel).org)\/(?P<board>\w+)\/thread\/(?P<opid>\d+)'
)
WEBHOOK_RE = re.compile(
    r'discord(?:app)?\.com/api/webhooks/(?P<id>[0-9]{17,20})/(?P<token>[A-Za-z0-9\.\-\_]{60,68})'
)

def validate_thread_url(url: str) -> Dict[str, Any]:
    """Validate a 4Chan thread url"""
    match = re.match(THREAD_RE, url)
    if match:
        return {
            'result': match.groups(),
            'success': True,
        }
    return {
        'success': False,
    }

def validate_webhook_url(url: str) -> Dict[str, Any]:
    """Validate a Discord webhook url"""
    m = re.search(
        WEBHOOK_RE, url
    )
    if m is None:
        return {
            'success': False,
        }
    return {
        'result': url,
        'success': True,
    }

def _construct_media_url(board: str, tim: str, ext: str) -> str:
    return f'https://i.4cdn.org/{board}/{tim}{ext}'

def construct_file(*, board: str, tim: str, filename: str, ext: str) -> discord.File:
    """
    Construct a discord.File object using board, tim, filename and ext.

    Params
    ------
    board: str
        The 4chan board name.
    tim: int
        The unique file 'tim'.
    filename: str
        The filename.
    ext: str
        The file extension.
    
    Returns
    -------
    A discord.File object.
    """
    url = _construct_media_url(board, tim, ext)
    resp = requests.get(url)
    if resp.status_code == 200:
        buffer = io.BytesIO(resp.content)
        return discord.File(buffer, f'{filename}{ext}')
    else:
        raise Exception('Invalid asset from 4chan', resp.status_code)

def normify_comment(comment: str) -> str:
    """
    Converts raw 4chan comments to readable format.
    This basically removes the unnecessary html tags and other stuff.

    Parameters
    ----------

    comment: str
        The comment to "normify".

    Returns
    -------
    The new edited string without the html tags.
    """
    RESTO_QUOTE_PATTERN = re.compile(r'(?:<a href=\"#p\d+\" class=\"quotelink\">&gt;&gt;\d+</a>)')

    fmt = re.sub(RESTO_QUOTE_PATTERN, '', comment) \
        .replace('<br>', '\n') \
        .replace('&#039;', "'") \
        .replace('&gt;', '>')
    return fmt